﻿-- Average Combat Mana ------------------------------------------------------

-- localise global variables for faster access ------------------------------
local tremove = tremove
local type = type
local UnitMaxPower = UnitMaxPower
local UnitPower = UnitPower
local wipe = wipe
local GetAddOnMetadata = GetAddOnMetadata
local tonumber = tonumber
local tostring = tostring
local floor = floor
local date = date
local SendChatMessge = SendChatMessage

-- declare addon ------------------------------------------------------------
local LibStub = _G.LibStub

local AverageCombatMana = LibStub("AceAddon-3.0"):NewAddon("AverageCombatMana", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("AverageCombatMana", true)

-- get version from .toc - set to Development if no version
local version = GetAddOnMetadata("AverageCombatMana", "Version")
--@alpha@
if version:match("@") then
	local MRG_Revision = tonumber(("$Revision: 12 $"):match("(%d+)"))
	version = "Development r"..tostring(MRG_Revision)
else
	version = "Alpha "..version
end
--@end-alpha@

-- add localisation to addon
AverageCombatMana.L = L

-- declare the database
local db

-- local variables ----------------------------------------------------------
local allManaValues = {} -- stores all values during combat
local ManaDisplay = CreateFrame("Frame", "ManaDisplay", UIParent)
--[[
ManaDisplay.segments = {}
ManaDisplay.message = {}
ManaDisplay.leftSide = {} 
ManaDisplay.rightSide = {}
ManaDisplay.indicator = {}
]]--

-- additional libraries -----------------------------------------------------
local DataBroker = LibStub("LibDataBroker-1.1")
-- local Graph = LibStub("LibGraph-2.0")
-- local Media = LibStub("LibSharedMedia-3.0")
-- local BackDrop = LibStub("LibBackdrop-1.0")

local defaults = {
	char = {
		logs = {}
	},
	profile = {
		addonEnabled = true,
		chatOutput = "1-self",
		showBar = true,
		-- graphBorder = "Blizzard Dialog Gold",
		-- graphTexture = "Blizzard",		
		-- displayGraph = true,		
	}
}

function AverageCombatMana:OnInitialize()
	-- register saved variables with AceDB
	db = LibStub("AceDB-3.0"):New("AverageCombatManaDB", defaults, "Default")
	db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	self.db = db
	self:SetEnabledState(self.db.profile.addonEnabled)
	
	-- addon options table	
	self.options = self:OptionsTable() -- see AverageCombatManaOptions.lua
	-- add the 'Profiles' section
	self.options.args.profilesTab = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	self.options.args.profilesTab.order = 50

	-- Register your options with AceConfigRegistry
	LibStub("AceConfig-3.0"):RegisterOptionsTable("AverageCombatMana", self.options)

	-- Add your options to the Blizz options window using AceConfigDialog
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("AverageCombatMana", "AverageCombatMana")	

	-- support for LibAboutPanel
	if LibStub:GetLibrary("LibAboutPanel", true) then
		self.optionsFrame[L["About"]] = LibStub("LibAboutPanel").new("AverageCombatMana", "AverageCombatMana")
	end	
		
	-- create the frame
	ManaDisplay = ManaDisplay or CreateFrame("Frame", "ManaDisplay", UIParent)
	ManaDisplay:SetPoint(self.db.profile.barPoint or "CENTER", self.db.profile.barX or 0, self.db.profile.barY or 250)
	ManaDisplay:SetHeight(20)
	ManaDisplay:SetWidth(210)
	
	ManaDisplay:SetBackdrop({
		bgFile = "Interface\\BUTTONS\\WHITE8X8", tile = false, tileSize = 16,
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,
		insets = { left = 4, right = 4, top = 4, bottom = 4 },
	})
	
	ManaDisplay:SetBackdropColor(0, 0, 0, 1)
	ManaDisplay:SetBackdropBorderColor( )
	
	ManaDisplay:SetAlpha(1)
	ManaDisplay.segments = ManaDisplay.segments or {}
	
	for i = -10, 10 do
		local t = ManaDisplay.segments[i] or ManaDisplay:CreateTexture(nil, "ARTWORK")
		t:SetPoint("LEFT", ManaDisplay.segments[i - 1], "RIGHT", 0, 0)
		t:SetWidth(1)
		t:SetHeight(ManaDisplay:GetHeight() - 10)
		t:SetTexture(TEXTURE)
		t:SetAlpha(0.8)
		ManaDisplay.segments[i] = t
	end
	
	-- mana loss
	for i = -10, -2 do
		local k = i/10
		ManaDisplay.segments[i]:SetVertexColor(k, 0, 0)
	end
	
	-- neither gain nor loss
	ManaDisplay.segments[-1]:SetVertexColor(0, 0, 0.5)
	ManaDisplay.segments[0]:SetVertexColor(0, 0, 1)
	ManaDisplay.segments[1]:SetVertexColor(0, 0, 0.5)
	
	-- mana gain
	for i = 2, 10 do
		local k = i/10
		ManaDisplay.segments[i]:SetVertexColor(0, k, 0)
	end
	
	ManaDisplay.message = ManaDisplay:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge")
	ManaDisplay.message:SetPoint("BOTTOM", ManaDisplay, "TOP", 0, 5)
	ManaDisplay.message:SetText("Average Combat Mana")

	ManaDisplay.leftSide = ManaDisplay:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	ManaDisplay.leftSide:SetPoint("TOPLEFT", ManaDisplay, "BOTTOMLEFT", 5, -5)
	ManaDisplay.leftSide:SetText(L["Full Burn!"])
	ManaDisplay.leftSide:SetTextColor(1, 0, 0, 1)

	ManaDisplay.rightSide = ManaDisplay:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	ManaDisplay.rightSide:SetPoint("TOPRIGHT", ManaDisplay, "BOTTOMRIGHT", -5, -5)
	ManaDisplay.rightSide:SetText(L["Gaining Mana!"])
	ManaDisplay.rightSide:SetTextColor(0, 1, 0, 1)

	ManaDisplay.indicator = ManaDisplay:CreateTexture(nil, "OVERLAY")
	ManaDisplay.indicator:SetPoint("CENTER", ManaDisplay.segments[0], "RIGHT")
	ManaDisplay.indicator:SetWidth(10)
	ManaDisplay.indicator:SetHeight(ManaDisplay:GetHeight() + 10)
	ManaDisplay.indicator:SetTexture("Interface\\WorldStateFrame\\WorldState-CaptureBar")
	ManaDisplay.indicator:SetTexCoord(0.8, 0.8, 0, 0.3)
	ManaDisplay.indicator:SetDesaturated(true)

	ManaDisplay:EnableMouse(true)
	ManaDisplay:SetMovable(true)
	ManaDisplay:SetClampedToScreen(true)
	ManaDisplay:RegisterForDrag("LeftButton")
	ManaDisplay:SetUserPlaced(false)
	ManaDisplay:SetScript("OnDragStart", function(self)
		if IsAltKeyDown() then
			ManaDisplay:StartMoving()
		end
	end)
	ManaDisplay:SetScript("OnDragStop", function(self)
		self:StopMovingOrSizing()
		self:SetUserPlaced(false)
		local point, _, _, x, y = self:GetPoint(1)
		AverageCombatMana.db.profile.barPoint = point
		AverageCombatMana.db.profile.barX = x
		AverageCombatMana.db.profile.barY = y
	end)
	
	--[[
	-- create real time graph
	self.RealTimeGraph = Graph:CreateGraphRealtime("TestRealtimeGraph",UIParent,"CENTER","CENTER",-90,90,150,150)
	self.RealTimeGraph:SetAutoScale(true)
	self.RealTimeGraph:SetGridSpacing(1, 10)
	]]--	

	-- add console commands
	self:RegisterChatCommand("acm", "SlashHandler")
	self:RegisterChatCommand("averagecombatmana", "SlashHandler")	
	
	-- create DataBroker Launcher
	if DataBroker then
		local launcher = DataBroker:NewDataObject("AverageCombatMana", {
			type = "launcher",
			icon = "Interface\\Icons\\INV_Potion_16",
			OnClick = function(clickedframe, button)
				if button == "LeftButton" then
					-- keep our options table in sync with the ldb object state
					--[[
					self.db.profile.displayGraph = not self.db.profile.displayGraph
					if self.db.profile.displayGraph then
						-- display graph code here
					else
						-- hide graph code here
					end
					LibStub("AceConfigRegistry-3.0"):NotifyChange("AverageCombatMana")
					]]--
				elseif button == "RightButton" then
					_G.InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
				--[[
				elseif button == "MiddleButton" then
					self:StartTest()
				]]--
				end
			end,
			OnTooltipShow = function(self)
			GameTooltip:AddLine("Average Combat Mana".." "..version, HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
			-- GameTooltip:AddLine(L["Left click to display real time graph."], NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
			-- GameTooltip:AddLine(L["Middle click to run test display."], NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
			GameTooltip:AddLine(L["Right click for configuration."], NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
			GameTooltip:Show()			
		end})
		self.launcher = launcher
	end
end

function AverageCombatMana:OnEnable()
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "NoCombat")
	self:RegisterEvent("PLAYER_REGEN_DISABLED", "Combat")
	-- Media.RegisterCallback(self, "OnValueChanged", "UpdateMedia")
	
	if self.db.profile.showBar and not ManaDisplay:IsShown() then
		ManaDisplay:Show()
	elseif not self.db.profile.showBar and ManaDisplay:IsShown() then
		ManaDisplay:Hide()
	end
end

function AverageCombatMana:OnDisable()
	self:UnregisterAllEvents()
	ManaDisplay:Hide()
	-- self.RealTimeGraph:ResetData()
	-- Media.UnregisterAllCallbacks(self)
end

function AverageCombatMana:OnProfileChanged()
	db = self.db
end

function AverageCombatMana:SlashHandler(input)
	_G.InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
end

--[[
function AverageCombatMana:UpdateMedia(callback, type, handle)
	-- local fontStyle
	if type == "border" then 
		ManaDisplay:SetBackdrop({ 
			edgeFile = Media:Fetch("border", self.db.profile.barBorder), 
			tile = false, 
			edgeSize = 16, 
			insets = { left = 4, right = 4, top = 4, bottom = 4 }, 
		})
	elseif type == "font" then
		if self.db.profile.fontEffects == "NONE" then
			fontStyle = nil
		end
		ManaDisplay.message:SetFont(profile.messageFont, 18, self.db.profile.fontEffects) 
		ManaDisplay.leftSide:SetFont(profile.manaFont, 14, self.db.profile.fontEffects) 
		ManaDisplay.rightSide:SetFont(profile.manaFont, 14, self.db.profile.fontEffects)
	end 
end
]]--

local in_combat
function AverageCombatMana:NoCombat()
	in_combat = false
	
	ManaDisplay:SetScript("OnUpdate", nil)
	
	if not self.db.profile.showBar and ManaDisplay:IsShown() then
		ManaDisplay:Hide()
	end
	-- reset indicator to posistion 0
	ManaDisplay.indicator:SetPoint("CENTER", ManaDisplay.segments[0], "RIGHT")
	ManaDisplay.leftSide:SetText(L["Full Burn!"])
	ManaDisplay.rightSide:SetText(L["Gaining Mana!"])
		
	-- calculate overall average gain or loss of Mana
	local allManaSum = 0
	
	local t = allManaValues
	for k = 1, #t do
		local v = t[k]
		allManaSum = allManaSum + v
	end
		
	--[===[@debug@
	self:Print("The sum of all combat Mana is ",allManaSum)
	--@end-debug@]===]
	local combatAverage = 0
	if allManaSum > 0 then
		combatAverage = allManaSum / #t
	end
	
	local function round(combatAverage, idp)
		local mult = 10 ^ (idp or 0)
		return floor(combatAverage * mult + 0.5) / mult
	end
	
	if self.db.profile.chatOutput ~= "0-none" then
		local chatType = self.db.profile.chatOutput:upper()
		if chatType == "2-smartGroup" then
			if UnitInRaid("player") then
				chatType = "RAID"
			elseif GetNumPartyMembers() > 0 then
				chatType = "PARTY"
			end
		end
		if self.db.profile.chatOutput == "1-self" then
			self:Print(L["Average Mana gain or loss for the entire fight was"], combatAverage)
		else
			SendChatMessage(L["Average Mana gain or loss for the entire fight was"].." %d", chatType, nil, nil, combatAverage)
		end
	end
	
	combatAverage = nil
end

function AverageCombatMana:Combat()
	in_combat = true
	local firstManaValue = UnitPower("player", SPELL_POWER_MANA)
	
	--[===[@debug@
	self:Print("The value of firstManaValue is ", firstManaValue)
	--@end-debug@]===]
		
	-- must show the display in combat, or nothing happens, including calculating/printing overall average
	if not ManaDisplay:IsShown() then
		ManaDisplay:Show()
	end
	ManaDisplay.leftSide:SetText("")
	ManaDisplay.rightSide:SetText("")
	
	-- get the 5 most recent UnitPower() values
	-- or wait until we actually get 5 of them
	-- the value of firstManaValue is retrieved when combat starts
	local counter = 0
	ManaDisplay:SetScript("OnUpdate", function(self, elapsed) 
		counter = counter + elapsed 
		if counter < 2 then return end
		counter = 0

		local currentMana = UnitPower("player", SPELL_POWER_MANA)
		
		--[===[@debug@
		AverageCombatMana:Print("The value of currentMana is ", currentMana)
		--@end-debug@]===]
		
		local deltaMana = currentMana - firstManaValue or currentMana
		firstManaValue = currentMana

		local numManaValues = #allManaValues + 1
		allManaValues[numManaValues] = deltaMana

		local count = numManaValues >= 5 and 5 or numManaValues
		for index = numManaValues - count + 1, numManaValues - 1 do
			deltaMana = deltaMana + allManaValues[index]
		end
		local average = deltaMana / count
		
		while average > 10 do
			average = average / 10
		end
		
		local function round(average, idp)
			local mult = 10 ^ (idp or 0)
			return floor(average * mult + 0.5) / mult
		end
		
		ManaDisplay.indicator:SetPoint("CENTER", ManaDisplay.segments[average], "RIGHT")
		
		if average < 0 then
			ManaDisplay.leftSide:SetFormattedText(L["Full Burn!\n%d"], average)
			ManaDisplay.rightSide:SetText("")
		elseif average > 1 then
			ManaDisplay.rightSide:SetFormattedText(L["Gaining Mana!\n%d"], average)
			ManaDisplay.leftSide:SetText("")
		else
			ManaDisplay.leftSide:SetText("")
			ManaDisplay.rightSide:SetText("")
		end
	end)
	
	self.db.char.logs[#self.db.char.logs + 1] = {
		date = date(),
		data = allManaValues,
	}		
	allManaValues = {}
end

function AverageCombatMana:ShowBar()
	if not ManaDisplay:IsShown() then
		ManaDisplay:Show()
	end
end

function AverageCombatMana:HideBar()
	if ManaDisplay:IsShown() then
		ManaDisplay:Hide()
	end
end